/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM /builds/tinderbox/XR-Trunk/Linux_2.6.18-8.el5_Depend/mozilla/dom/public/idl/offline/nsIDOMOfflineResourceList.idl
 */

#ifndef __gen_nsIDOMOfflineResourceList_h__
#define __gen_nsIDOMOfflineResourceList_h__


#ifndef __gen_domstubs_h__
#include "domstubs.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    nsIDOMOfflineResourceList */
#define NS_IDOMOFFLINERESOURCELIST_IID_STR "8449bce2-0d8c-4c74-ab79-b41b8d81f1c4"

#define NS_IDOMOFFLINERESOURCELIST_IID \
  {0x8449bce2, 0x0d8c, 0x4c74, \
    { 0xab, 0x79, 0xb4, 0x1b, 0x8d, 0x81, 0xf1, 0xc4 }}

class NS_NO_VTABLE nsIDOMOfflineResourceList : public nsISupports {
 public: 

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IDOMOFFLINERESOURCELIST_IID)

  /* readonly attribute unsigned long length; */
  NS_IMETHOD GetLength(PRUint32 *aLength) = 0;

  /* DOMString item (in unsigned long index); */
  NS_IMETHOD Item(PRUint32 index, nsAString & _retval) = 0;

  /* void add (in DOMString uri); */
  NS_IMETHOD Add(const nsAString & uri) = 0;

  /* void remove (in DOMString uri); */
  NS_IMETHOD Remove(const nsAString & uri) = 0;

  /* boolean has (in DOMString uri); */
  NS_IMETHOD Has(const nsAString & uri, PRBool *_retval) = 0;

  /* void clear (); */
  NS_IMETHOD Clear(void) = 0;

  /* void refresh (); */
  NS_IMETHOD Refresh(void) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIDOMOfflineResourceList, NS_IDOMOFFLINERESOURCELIST_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIDOMOFFLINERESOURCELIST \
  NS_IMETHOD GetLength(PRUint32 *aLength); \
  NS_IMETHOD Item(PRUint32 index, nsAString & _retval); \
  NS_IMETHOD Add(const nsAString & uri); \
  NS_IMETHOD Remove(const nsAString & uri); \
  NS_IMETHOD Has(const nsAString & uri, PRBool *_retval); \
  NS_IMETHOD Clear(void); \
  NS_IMETHOD Refresh(void); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIDOMOFFLINERESOURCELIST(_to) \
  NS_IMETHOD GetLength(PRUint32 *aLength) { return _to GetLength(aLength); } \
  NS_IMETHOD Item(PRUint32 index, nsAString & _retval) { return _to Item(index, _retval); } \
  NS_IMETHOD Add(const nsAString & uri) { return _to Add(uri); } \
  NS_IMETHOD Remove(const nsAString & uri) { return _to Remove(uri); } \
  NS_IMETHOD Has(const nsAString & uri, PRBool *_retval) { return _to Has(uri, _retval); } \
  NS_IMETHOD Clear(void) { return _to Clear(); } \
  NS_IMETHOD Refresh(void) { return _to Refresh(); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIDOMOFFLINERESOURCELIST(_to) \
  NS_IMETHOD GetLength(PRUint32 *aLength) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetLength(aLength); } \
  NS_IMETHOD Item(PRUint32 index, nsAString & _retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->Item(index, _retval); } \
  NS_IMETHOD Add(const nsAString & uri) { return !_to ? NS_ERROR_NULL_POINTER : _to->Add(uri); } \
  NS_IMETHOD Remove(const nsAString & uri) { return !_to ? NS_ERROR_NULL_POINTER : _to->Remove(uri); } \
  NS_IMETHOD Has(const nsAString & uri, PRBool *_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->Has(uri, _retval); } \
  NS_IMETHOD Clear(void) { return !_to ? NS_ERROR_NULL_POINTER : _to->Clear(); } \
  NS_IMETHOD Refresh(void) { return !_to ? NS_ERROR_NULL_POINTER : _to->Refresh(); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsDOMOfflineResourceList : public nsIDOMOfflineResourceList
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIDOMOFFLINERESOURCELIST

  nsDOMOfflineResourceList();

private:
  ~nsDOMOfflineResourceList();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsDOMOfflineResourceList, nsIDOMOfflineResourceList)

nsDOMOfflineResourceList::nsDOMOfflineResourceList()
{
  /* member initializers and constructor code */
}

nsDOMOfflineResourceList::~nsDOMOfflineResourceList()
{
  /* destructor code */
}

/* readonly attribute unsigned long length; */
NS_IMETHODIMP nsDOMOfflineResourceList::GetLength(PRUint32 *aLength)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* DOMString item (in unsigned long index); */
NS_IMETHODIMP nsDOMOfflineResourceList::Item(PRUint32 index, nsAString & _retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void add (in DOMString uri); */
NS_IMETHODIMP nsDOMOfflineResourceList::Add(const nsAString & uri)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void remove (in DOMString uri); */
NS_IMETHODIMP nsDOMOfflineResourceList::Remove(const nsAString & uri)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* boolean has (in DOMString uri); */
NS_IMETHODIMP nsDOMOfflineResourceList::Has(const nsAString & uri, PRBool *_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void clear (); */
NS_IMETHODIMP nsDOMOfflineResourceList::Clear()
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void refresh (); */
NS_IMETHODIMP nsDOMOfflineResourceList::Refresh()
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIDOMOfflineResourceList_h__ */
